@@ -56,7 +56,7 @@ class ConfigClass:
         return s
 
     def load(self, filename=None):
-        filename = filename or os.path.abspath(os.path.join(os.path.dirname(__file__), '../config/config.yaml'))
+        filename = filename or os.path.abspath(os.path.join(os.path.dirname(__file__), 'config.yaml'))
         with open(filename, 'r') as fp:
             conf = yaml.load(fp, yaml.FullLoader)
 
@@ -71,5 +71,11 @@ class ConfigClass:
     def __getattr__(self, attrname):
         return None
 
+    def __contains__(self, key):
+        return key in self.__dict__
+
+    def get(self, key, default=None):
+        return self.__dict__.get(key, default)
+
 
 Config = ConfigClass()
