@@ -114,7 +114,7 @@ class Storage:
                         asyncio.create_task(sub.notify(event))
                         count += 1
             if count:
-                LOG.info("notify-all took %.2fms for %d subscriptions", t(), count)
+                LOG.debug("notify-all took %.2fms for %d subscriptions", t(), count)
         return event, changed
 
     def validate_event(self, event):
@@ -171,7 +171,8 @@ class Storage:
                 # update mentions
                 # single-letter tags can be searched
                 # delegation tags are also searched
-                tags = set((event.id_bytes, tag[0], tag[1]) for tag in event.tags if tag[0] == 'delegation' or len(tag[0]) == 1)
+                # expiration tags are also added for the garbage collector
+                tags = set((event.id_bytes, tag[0], tag[1]) for tag in event.tags if tag[0] in ('delegation', 'expiration') or len(tag[0]) == 1)
                 if tags:
                     await cursor.executemany('INSERT OR IGNORE INTO tag (id, name, value) VALUES (?, ?, ?)', tags)
             elif event.kind == EventKind.DELETE and event.tags:
@@ -188,8 +189,6 @@ class Storage:
         LOG.debug('%s/%s filters: %s', client_id, sub_id, filters)
         if sub_id in self.clients[client_id]:
             await self.unsubscribe(client_id, sub_id)
-            # rate limit on resubscribing
-            await asyncio.sleep(0.75)
         sub = Subscription(self.db, sub_id, filters, queue=queue, client_id=client_id)
         if sub.prepare():
             asyncio.create_task(sub.run_query())
@@ -204,9 +203,10 @@ class Storage:
                 LOG.info("%s/%s -", client_id, sub_id)
             except KeyError:
                 pass
-        else:
+        elif client_id in self.clients:
             del self.clients[client_id]
 
+
     async def num_subscriptions(self, byclient=False):
         subs = {}
         for client_id, client in self.clients.items():
@@ -337,25 +337,25 @@ class Subscription:
 
     def check_event(self, event, filters):
         for filter_obj in filters:
-            matched = []
+            matched = set()
             for key, value in filter_obj.items():
                 if key == 'ids':
-                    matched.append(bool(event.id in value))
+                    matched.add(event.id in value)
                 elif key == 'authors':
-                    matched.append(bool(event.pubkey in value))
+                    matched.add(event.pubkey in value)
                     for tag in event.tags:
                         if tag[0] == 'delegation' and tag[1] in value:
-                            matched.append(True)
+                            matched.add(True)
                 elif key == 'kinds':
-                    matched.append(bool(event.kind in value))
+                    matched.add(event.kind in value)
                 elif key == 'since':
-                    matched.append(bool(event.created_at >= value))
+                    matched.add(event.created_at >= value)
                 elif key == 'until':
-                    matched.append(bool(event.created_at < value))
+                    matched.add(event.created_at < value)
                 elif key[0] == '#' and len(key) == 2:
                     for tag in event.tags:
                         if tag[0] == key[1]:
-                            matched.append(bool(tag[1] in value))
+                            matched.add(tag[1] in value)
             if all(matched):
                 return True
         return False
@@ -498,12 +498,20 @@ class BaseGarbageCollector(threading.Thread):
 
 class QueryGarbageCollector(BaseGarbageCollector):
     query = '''
-        DELETE FROM event WHERE kind >= 20000 and kind < 30000;
+        DELETE FROM event WHERE event.id IN
+        (
+            SELECT event.id FROM event
+            LEFT JOIN tag on tag.id = event.id
+            WHERE 
+                (kind >= 20000 and kind < 30000)
+            OR
+                (tag.name = 'expiration' AND tag.value < strftime('%s'))
+    )
     '''
 
     def collect(self, db):
         cursor = db.cursor()
-        cursor.executescript(self.query)
+        cursor.execute(self.query)
         cursor.close()
         return max(0, cursor.rowcount)
 
